#include <stm32f10x_lib.h>



#define AdresseCapteur 0x94	/*Adresse de commande du capteur de temprature TMP100*/



void Delai(int t);
void Initialisation_I2C_1(void);
int LirePlusieursOctets(unsigned char adresse, unsigned char nombre, unsigned char receptacle[]);
int Lire16Nouvelle(unsigned char adresse, unsigned char Commande, unsigned char adresseDeLecture, unsigned char * Donnee1, unsigned char * Donnee2);
int EcrirePlusieursOctets(unsigned char adresse, unsigned char nombre, unsigned char donnees[]);
int Ecrire(unsigned char adresse, unsigned char donnee);
int Ecrire16Court(unsigned char adresse, unsigned char commande, unsigned char donnee);
int Ecrire16Nouvelle(unsigned char adresse, unsigned char commande, unsigned char donnee1, unsigned char donnee2);
int Configuration_pour_capteur(unsigned char a);



/*Fonction qui sert  faire perdre son temps au STM32F100RB

Paramtre d'entre :

t - Valeur qui est proportionnelle au dlai que l'on souhaite obtenir.

Valeur de retour :

Aucune.

*/
void Delai(int t)
	{
		int i;
		for(i = 0; i < (10 * t); i++);
	}



/*Initialisation du module  I2C 1  du STM32F100RB en tant que  Matre 

Paramtre d'entre :

Aucun.

Valeur de retour :

Aucune.

*/
void Initialisation_I2C_1(void)
	{
		/* GPIOB clock enable */
		RCC->APB2ENR |= (1 << 3);

        /* I2C1 clock enable */
		RCC->APB1ENR |= (1 << 21);

        /* I2C1 SDA and SCL configuration */
		GPIOB->CRL = ((GPIOB->CRL & 0x00FFFFFF) | (0xFF000000));

        /* Enable I2C1 reset state */
		RCC->APB1RSTR |= (1 << 21);
        /* Release I2C1 from reset state */
		RCC->APB1RSTR &= ~(1 << 21);

		I2C1->CR1 = 0x0401;
		I2C1->CR2 = 0x0108;
		
		I2C1->SR1 = 0x0000;
		I2C1->SR2 = 0x0000;
		
		I2C1->CCR = 0x8006;
		
		I2C1->OAR1 = 0x4028;
		I2C1->OAR2 = 0x0000;
		
		I2C1->TRISE = 0x0003;
	}



/*Lecture d'un ou de plusieurs octets grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

adresse - Adresse de lecture, sur le bus  I2C 1  du composant que l'on souhaite lire,
nombre - Nombre d'octets  lire,
receptacle[] - Tableau qui est destin  contenir le ou les octets qui auront t lus.

Valeur de retour :

1 si la lecture s'est effectue correctement, -1 s'il y a eu une erreur lors de la lecture.

*/
int LirePlusieursOctets(unsigned char adresse, unsigned char nombre, unsigned char receptacle[])
	{
	    int temp = 0;
	    int Timeout = 0;
			if(nombre <= 2)
		  		{
		           /* Set POS bit */
		            I2C1->CR1 |= 0x0800;
		            Timeout = 0xFFFF;
		            /* Send START condition */
		            I2C1->CR1 |= 0x0100;
		            /* Wait until SB flag is set: EV5 */
			            while ((I2C1->SR1&0x0001) != 0x0001)
				            {
				                if (Timeout-- == 0)
				                    return (-1);
				            }
		            Timeout = 0xFFFF;
		            /* Send the slave address */
		            I2C1->DR = adresse;
		            /* Wait until ADDR is set: EV6 */
			            while((I2C1->SR1&0x0002) != 0x0002)
				            {
				                if(Timeout-- == 0)
				                    return (-1);
				            }
		            /* Clear ADDR by reading SR2 register  */
		            temp = I2C1->SR2;
		            /* Clear ACK */
		            I2C1->CR1 &= 0xFBFF;
		            /* Wait until BTF is set */
		            	while((I2C1->SR1 & 0x00004) != 0x000004);
		            /* Program the STOP */
		            I2C1->CR1 |= 0x0200;
		            /* Read first data */
		            receptacle[0] = I2C1->DR;
						if(nombre == 2)
							{
					            /* Read second data */
					            receptacle[1] = I2C1->DR;
							}
			            /* Make sure that the STOP bit is cleared by Hardware before CR1 write access */
			            while ((I2C1->CR1&0x200) == 0x200);
		            /* Enable Acknowledgement to be ready for another reception */
		            I2C1->CR1  |= 0x0400;
		            /* Clear POS bit */
		            I2C1->CR1  &= 0xF7FF;
				}
			return 1;
	}



/*Lecture de deux octets grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

adresse - Adresse d'criture, sur le bus  I2C 1 , du composant que l'on souhaite lire,
Commande - Numro du registre que l'on souhaite lire,
adresseDeLecture - Adresse de lecture, sur le bus  I2C 1  du composant que l'on souhaite lire,
* Donne 1 - Pointeur sur une variable, de type  unsigned char , destine  contenir la valeur du premier octet  lire,
* Donne 2 - Pointeur sur une variable, de type  unsigned char , destine  contenir la valeur du deuxime octet  lire.

Valeur de retour :

1 si la lecture s'est effectue correctement, -1 s'il y a eu une erreur lors de la lecture.

*/
int Lire16Nouvelle(unsigned char adresse, unsigned char Commande, unsigned char adresseDeLecture, unsigned char * Donnee1, unsigned char * Donnee2)
	{
		unsigned char tampon[2];
			if(Ecrire(adresse, Commande) == (-1))	/*On indique, dans un premier temps, le numro du registre dont on souhaite lire le contennu*/
				{
				 	return (-1);
				}
			if(LirePlusieursOctets(adresseDeLecture, 2, tampon) == (-1))	/*On lit, dans un second temps, les deux octets du registre slectionn*/
				{
				 	return (-1);
				}
			*Donnee1 = tampon[0];
			*Donnee2 = tampon[1];
		return 1;
	}



/*Ecriture d'un ou de plusieurs octets grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

adresse - Adresse d'criture, sur le bus  I2C 1 , du composant que l'on souhaite crire,
nombre - Nombre d'octets  crire,
donnees[] - Tableau qui contient le ou les octets  crire.

Valeur de retour :

1 si l'criture s'est effectue correctement, -1 s'il y a eu une erreur lors de l'criture.

*/
int EcrirePlusieursOctets(unsigned char adresse, unsigned char nombre, unsigned char donnees[]) 
	{
		int temp = 0;
	    int Timeout = 0;
		int p = 0;
        Timeout = 0xFFFF;
        /* Send START condition */
        I2C1->CR1 |= 0x0100;
	        /* Wait until SB flag is set: EV5 */
	        while ((I2C1->SR1&0x0001) != 0x0001)
		        {
		            if (Timeout-- == 0)
						{
		 					Initialisation_I2C_1();
			                	return (-1);
						}
		        }
        /* Send the slave address */
        I2C1->DR = adresse;
        Timeout = 0xFFFF;
	        /* Wait until ADDR is set: EV6 */
	        while ((I2C1->SR1 &0x0002) != 0x0002)
		        {
		            if (Timeout-- == 0)
						{
		 					Initialisation_I2C_1();
			                	return (-1);
						}
		        }
        /* Clear ADDR flag by reading SR2 register */
        temp = I2C1->SR2;
        /* Write the first data in DR register (EV8_1) */
        I2C1->DR = donnees[0];
        /* Increment */
        p++;
        /* Decrement the number of bytes to be written */
        nombre--;
	        /* While there is data to be written */
	        while(nombre--)
		        {
		            /* Poll on BTF to receive data because in polling mode we can not guarantee the
		              EV8 software sequence is managed before the current byte transfer completes */
		            while ((I2C1->SR1 & 0x00004) != 0x000004);
			            /* Send the current byte */
			            I2C1->DR = donnees[p];
			            /* Point to the next byte to be written */
			            p++;
		        }
	        /* EV8_2: Wait until BTF is set before programming the STOP */
	        while ((I2C1->SR1 & 0x00004) != 0x000004);
	        /* Send STOP condition */
	    I2C1->CR1 |= 0x0200;
	        /* Make sure that the STOP bit is cleared by Hardware */
	        while ((I2C1->CR1&0x200) == 0x200);

	    	return 1;
	}



/*Ecriture d'un octet grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

adresse - Adresse d'criture, sur le bus  I2C 1 , du composant que l'on souhaite crire,
donnees - Donne  crire.

Valeur de retour :

1 si l'criture s'est effectue correctement, -1 s'il y a eu une erreur lors de l'criture.

*/
int Ecrire(unsigned char adresse, unsigned char donnee)
	{
	 	unsigned char tampon[1];
			tampon[0] = donnee;
		return (EcrirePlusieursOctets(adresse, 1, tampon));
	}



/*Ecriture d'un octet, dans un registre d'un composant, grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

adresse - Adresse d'criture, sur le bus  I2C 1 , du composant que l'on souhaite crire,
commande - Numro du registre dans lequel on souhaite crire,
donnees - Donne  crire.

Valeur de retour :

1 si l'criture s'est effectue correctement, -1 s'il y a eu une erreur lors de l'criture.

*/
int Ecrire16Court(unsigned char adresse, unsigned char commande, unsigned char donnee) 
	{
	 	unsigned char tampon[2];
			tampon[0] = commande;
			tampon[1] = donnee;
		return (EcrirePlusieursOctets(adresse, 2, tampon));
	}



/*Ecriture de deux octets, dans un registre d'un composant, grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

adresse - Adresse d'criture, sur le bus  I2C 1 , du composant que l'on souhaite crire,
commande - Numro du registre dans lequel on souhaite crire,
donnees1 - Premier octet  crire, l'octet de poids fort,
donnees2 - Deuxime octet  crire, l'octet de poids faible.

Valeur de retour :

1 si l'criture s'est effectue correctement, -1 s'il y a eu une erreur lors de l'criture.

*/
int Ecrire16Nouvelle(unsigned char adresse, unsigned char commande, unsigned char donnee1, unsigned char donnee2)
	{
	 	unsigned char tampon[3];
			tampon[0] = commande;
			tampon[1] = donnee1;
			tampon[2] = donnee2;
		return (EcrirePlusieursOctets(adresse, 3, tampon));
	}



/*Configurer un capteur de temprature TMP100 

Paramtre d'entre :

a - Adresse d'criture, sur le bus  I2C 1 , du capteur de temprature TMP100 que l'on souhaite configurer.

Valeur de retour :

1 si la configuration s'est effectue correctement, -1 s'il y a eu une erreur lors de la configuration.

*/
int Configuration_pour_capteur(unsigned char a)
	{
		if((Ecrire16Court(a, 0x01, 0x60)) == (-1))	/*Ecriture dans le  Configuration Register  :
												SD = 0 : Dsactivation du  Shutdown Mode ,
												TM = 0 : Non utilis pour le TMP100,
												POL = 0 : Non utilis pour le TMP100,
												F0 = 0, F1 = 0 : Non utilis pour le TMP100,
												R0 = 1, R1 = 1 : Choix de la rsolution maximale,
												pour la conversion de la temprature, soit sur 12 bits, par pas de 0.0625C,
												ce qui signifie que le temps de conversion est de 320 ms*/
			{
				return (-1);
			}
					return 1;
	}



/*Fonction principale 

Paramtre d'entre :

Aucun.

Valeur de retour :

Aucune.

*/
int main (void)
	{
		unsigned char E1 = 0;
		unsigned char E2 = 0;

    	Initialisation_I2C_1();	/*Initialisation du module  I2C 1  du STM32F100RB en tant que  Matre */

		Configuration_pour_capteur(AdresseCapteur);
			 
			while(1)
				{
					Lire16Nouvelle(AdresseCapteur, 0x00, (AdresseCapteur | 0x01), &E1, &E2);	/*Lecture des deux registres de 8 bits
																								qui contiennent la valeur de
																								la temprature mesure*/
																								
					Delai(10000);																
				}
	}



/*Fonction qui permet l'initialisation du microcontrleur STM32F100RB

Paramtre d'entre :

Aucun.

Valeur de retour :

Aucune.

*/
void SystemInit (void)
	{
	
	}
